"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const yosay = require("yosay");
const ejs = require("ejs");
const base_1 = require("../../lib/base");
const makeup_1 = require("./makeup");
class AppGenerator extends base_1.default {
    constructor(args, options) {
        super(args, options);
        this.sourceRoot(path.join(__dirname, "../../../templates/app"));
        this._setupMakeup();
    }
    _setupMakeup() {
        const makeup = new makeup_1.default(this);
        makeup.applyConfig(this.config.getAll());
        makeup.applyOptions(this.options);
        makeup.applyDefaults(this.appname);
        this.makeup = makeup;
    }
    prompting() {
        if (!this.options["skip-welcome-message"]) {
            // Have Yeoman greet the user.
            this.log(yosay("Welcome to the marvelous WebpackVersatile generator!"));
        }
        const questions = this.makeup.getQuestions();
        return this.prompt(questions)
            .then((answers) => {
            this.makeup.putAnswers(answers);
        });
    }
    configuring() {
        this.config.set(this.makeup.getPermanetConfig(), undefined);
        this.config.save();
    }
    writing() {
        const templateConfig = this.makeup.getTemplateConfig();
        const dirs = templateConfig.dirs;
        const templateTransformer = {
            src: /^%(.*)/,
            tgt: "$1",
            fn: (s) => {
                return ejs.render(s, templateConfig);
            },
        };
        // this.log('copy project files');
        this._branchCopy({
            source: "root",
            branches: templateConfig.use,
            transformers: [templateTransformer],
        });
        // this.log('copy src files');
        this._branchCopy({
            source: "src",
            target: dirs.src,
            branches: templateConfig.use,
            transformers: [templateTransformer],
        });
        // this.log('copy test files');
        this._branchCopy({
            source: "test",
            target: dirs.test,
            branches: templateConfig.use,
            transformers: [templateTransformer],
        });
    }
    install() {
        if (!this.options["skip-install"]) {
            this.installDependencies();
        }
    }
}
exports.default = AppGenerator;
